// -*- mode: js; js-indent-level: 4; indent-tabs-mode: nil -*-
/*
  Copyright (C) 2012-2019  Dieter Schärf <ds@dschaerf.de>

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

const Lang = imports.lang;

const ExtensionUtils = imports.misc.extensionUtils;
const Main = imports.ui.main;
const StatusAccessibility = imports.ui.status.accessibility;

const Me = ExtensionUtils.getCurrentExtension();

const Convenience = Me.imports.convenience;

const KEY_SHOW_ACCESSIBILITY = 'show-accessibility';

const PANELMENU_BUTTON_DEFAULT = 0;
const PANELMENU_BUTTON_SHOW = 1;

const AccessibilityMenuButton = new Lang.Class({
    Name: 'PanelManager.AccessibilityMenuButton',

    _init: function() {
        this._changedAccessibility(true);
        this._settingsChangedId = Convenience.Settings.connect('changed::'+KEY_SHOW_ACCESSIBILITY, Lang.bind(this, this._onShowAccessibility));
    },

    _changedAccessibility: function(enabled) {
        /**
         *  Typing error in accessibility.js in function _queueSyncMenuVisibility:
         *  - this._syncMenuVisbilityIdle instead of this._syncMenuVisibilityIdle
         */
        let typingError = Convenience.checkVersion([ '3.18', '3.20', '3.22' ]);
        if (enabled) {
            if (typingError) Main.panel.statusArea.a11y._queueSyncMenuVisibility = function() {
                if (this._syncMenuVisibilityIdle) return;
                this._syncMenuVisibilityIdle = Mainloop.idle_add(Lang.bind(this, this._syncMenuVisibility));
                GLib.Source.set_name_by_id(this._syncMenuVisibilityIdle, '[gnome-shell] this._syncMenuVisibility');
            };
            Main.panel.statusArea.a11y._syncMenuVisibility = function() {
                let pmReturn = StatusAccessibility.ATIndicator.prototype._syncMenuVisibility.call(this);
                let pmShowButton = Convenience.Settings.get_enum(KEY_SHOW_ACCESSIBILITY);
                if (pmShowButton !== PANELMENU_BUTTON_DEFAULT) this.actor.visible = pmShowButton === PANELMENU_BUTTON_SHOW;
                return pmReturn;
            };
        } else {
            Main.panel.statusArea.a11y._syncMenuVisibility = StatusAccessibility.ATIndicator.prototype._syncMenuVisibility;
            if (typingError) Main.panel.statusArea.a11y._queueSyncMenuVisibility = StatusAccessibility.ATIndicator.prototype._queueSyncMenuVisibility;
        }
        this._onShowAccessibility();
    },

    _onShowAccessibility: function() {
        Main.panel.statusArea.a11y._syncMenuVisibility.call(Main.panel.statusArea.a11y);
    },

    destroy: function() {
        Convenience.Settings.disconnect(this._settingsChangedId);
        this._changedAccessibility(false);
    }
});

function getExtension() {
    return new AccessibilityMenuButton();
}
