// -*- mode: js; js-indent-level: 4; indent-tabs-mode: nil -*-
/*
  Copyright (C) 2012-2019  Dieter Schärf <ds@dschaerf.de>

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

const Lang = imports.lang;
const St = imports.gi.St;

const ExtensionUtils = imports.misc.extensionUtils;
const Main = imports.ui.main;

const Me = ExtensionUtils.getCurrentExtension();

const Convenience = Me.imports.convenience;

const DateMenuButton = new Lang.Class({
    Name: 'PanelManager.DateMenuButton',

    _init: function() {
        this._changedDateMenu(true);
        this._eventsChangedId = [
            Main.panel.statusArea.dateMenu._messageList._eventsSection._list.connect('actor-added', Lang.bind(this, this._updateVisibility)),
            Main.panel.statusArea.dateMenu._messageList._eventsSection._list.connect('actor-removed', Lang.bind(this, this._updateVisibility))
        ];
        this._notificationsChangedId = [
            Main.panel.statusArea.dateMenu._messageList._notificationSection._list.connect('actor-added', Lang.bind(this, this._updateVisibility)),
            Main.panel.statusArea.dateMenu._messageList._notificationSection._list.connect('actor-removed', Lang.bind(this, this._updateVisibility))
        ];
        this._updateVisibility();
    },

    _changedDateMenu: function(enabled) {
        let box = Main.panel.statusArea.dateMenu.actor.get_children()[0];
        if (enabled) {
            this._iconEvents = new St.Icon({ icon_name: 'x-office-calendar-symbolic', style_class: 'system-status-icon' });
            box.insert_child_at_index(this._iconEvents, 0);
            this._iconNotification = new St.Icon({ icon_name: 'dialog-warning-symbolic', style_class: 'system-status-icon' });
            box.insert_child_at_index(this._iconNotification, 0);
            box.add_style_class_name('panel-status-menu-box');
        } else {
            box.remove_style_class_name('panel-status-menu-box');
            box.remove_actor(this._iconNotification);
            box.remove_actor(this._iconEvents);
            Main.panel.statusArea.dateMenu._indicator._updateCount();
        }
    },

    _updateVisibility: function() {
        Main.panel.statusArea.dateMenu._indicator.actor.hide();
        let numEvents = Main.panel.statusArea.dateMenu._messageList._eventsSection._list.get_children().length > 0;
        let numNotification = Main.panel.statusArea.dateMenu._messageList._notificationSection._list.get_children().length > 0;
        this._iconEvents.visible = numEvents;
        this._iconNotification.visible = numNotification;
    },

    destroy: function() {
        this._notificationsChangedId.forEach(function(id) {
            Main.panel.statusArea.dateMenu._messageList._notificationSection._list.disconnect(id);
        });
        this._eventsChangedId.forEach(function(id) {
            Main.panel.statusArea.dateMenu._messageList._eventsSection._list.disconnect(id);
        });
        this._changedDateMenu(false);
    }
});

function getExtension() {
    return new DateMenuButton();
}
