// -*- mode: js; js-indent-level: 4; indent-tabs-mode: nil -*-
/*
  Copyright (C) 2012-2019  Dieter Schärf <ds@dschaerf.de>

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

const Lang = imports.lang;
const St = imports.gi.St;

const ExtensionUtils = imports.misc.extensionUtils;
const Main = imports.ui.main;
const PopupMenu = imports.ui.popupMenu;

const Me = ExtensionUtils.getCurrentExtension();

const Convenience = Me.imports.convenience;

const KEY_SHOW_SYSTEM_MENU = 'show-systemmenu';

const SYSTEM_MENU_DEFAULT = 0;
const SYSTEM_MENU_ICON = 1;

const SYSTEM_MENU_EXTENSIONS = [
    'systemMenuVolume',
    'systemMenuSystem',
];

function _getExtension(extension) {
    //try { return Me.imports[extension].getExtension(); } catch(e) { return null; }
    return Me.imports[extension].getExtension();
}

const SystemMenuButton = new Lang.Class({
    Name: 'PanelManager.SystemMenuButton',

    _init: function() {
        this._changedSystemMenu(true);
        this._settingsChangedId = Convenience.Settings.connect('changed::'+KEY_SHOW_SYSTEM_MENU, Lang.bind(this, this._onShowSystemMenu));
        SYSTEM_MENU_EXTENSIONS.forEach(function(extension) {
            this[extension] = _getExtension(extension);
        }, this);
    },

    _changedSystemMenu: function(enabled) {
        if (enabled) {
            this._hbox = new St.BoxLayout({ style_class: 'panel-status-menu-box' });
            Main.panel.statusArea.aggregateMenu.actor.remove_actor(Main.panel.statusArea.aggregateMenu._indicators);
            this._hbox.add_child(Main.panel.statusArea.aggregateMenu._indicators);
            this._indicators = new St.BoxLayout({ style_class: 'panel-status-menu-box' });
            this._indicators.add_child(new St.Icon({ icon_name: 'system-shutdown-symbolic', style_class: 'system-status-icon' }));
            this._indicators.add_child(PopupMenu.arrowIcon(St.Side.BOTTOM));
            this._hbox.add_child(this._indicators);
            Main.panel.statusArea.aggregateMenu.actor.add_child(this._hbox);
            this._separatorSystem = new PopupMenu.PopupSeparatorMenuItem();
            Main.panel.statusArea.aggregateMenu.menu.addMenuItem(this._separatorSystem, Main.panel.statusArea.aggregateMenu.menu.numMenuItems - 1);
            this._onShowSystemMenu();
        } else {
            this._separatorSystem.destroy();
            this._hbox.remove_actor(Main.panel.statusArea.aggregateMenu._indicators);
            Main.panel.statusArea.aggregateMenu.actor.remove_actor(this._hbox);
            Main.panel.statusArea.aggregateMenu.actor.add_child(Main.panel.statusArea.aggregateMenu._indicators);
        }
    },

    _onShowSystemMenu: function() {
        let showSystemMenu = Convenience.Settings.get_enum(KEY_SHOW_SYSTEM_MENU);
        Main.panel.statusArea.aggregateMenu._indicators.visible = showSystemMenu === SYSTEM_MENU_DEFAULT;
        this._indicators.visible = showSystemMenu === SYSTEM_MENU_ICON;
    },

    destroy: function() {
        SYSTEM_MENU_EXTENSIONS.forEach(function(extension) {
            if (this[extension] !== null) this[extension].destroy();
        }, this);
        Convenience.Settings.disconnect(this._settingsChangedId);
        this._changedSystemMenu(false);
    }
});

function getExtension() {
    return new SystemMenuButton();
}
